import { Injectable } from '@nestjs/common';
import { Repository } from 'typeorm';
import { Pair } from './entities/pair.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { PairMember } from './entities/pair-member.entity';
import { UsersService } from 'src/users/users.service';
import {
  UserIdNotExistsException,
  UserNotBelongsToPair,
} from 'src/utils/errors';
import { Swipe } from 'src/swipes/entities/swipe.entity';

@Injectable()
export class PairsService {
  constructor(
    @InjectRepository(Pair)
    private readonly pairsRepository: Repository<Pair>,
    @InjectRepository(PairMember)
    private readonly pairMembersRepository: Repository<PairMember>,
    private readonly usersService: UsersService,
  ) {}

  async createPair(userIds: [number, number]) {
    const users = await this.usersService.getByIds(userIds);
    if (users.length !== 2) {
      throw new UserIdNotExistsException();
    }

    const pair = await this.pairsRepository.manager.transaction(
      async (manager) => {
        const pair = this.pairsRepository.create();
        await manager.save(pair);

        const pairMembers = this.pairMembersRepository.create([
          {
            pairId: pair.id,
            userId: userIds[0],
          },
          {
            pairId: pair.id,
            userId: userIds[1],
          },
        ]);
        await manager.save(pairMembers);

        return pair;
      },
    );

    return pair;
  }

  async destroyPair(userId: number, pairId: number) {
    const pair = await this.pairsRepository.findOne({
      where: { id: pairId },
      relations: { pairMembers: { user: true } },
    });
    const members = pair.pairMembers;
    if (!members.some((member: PairMember) => member.user.id === userId)) {
      throw new UserNotBelongsToPair();
    }
    const anotherMember = members.find(
      (member: PairMember) => member.user.id !== userId,
    );

    await this.pairsRepository.manager.transaction(async (manager) => {
      await manager.delete(PairMember, { pair });
      await manager.delete(Pair, { id: pairId });
      await manager.update(
        Swipe,
        {
          sourceUser: { id: userId },
          targetUser: { id: anotherMember.user.id },
        },
        { liked: false },
      );
    });
  }
}
